@@ -3,6 +3,7 @@ use std::fmt;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::io;
+use std::mem;
 use std::mem::MaybeUninit;
 use std::os::raw::c_int;
 use std::os::unix::process::ExitStatusExt;
@@ -11,12 +12,14 @@ use std::process::Child;
 use std::thread;
 use std::time::Duration;
 
-#[cfg(target_env = "gnu")]
+#[cfg(all(target_env = "gnu", target_os = "linux"))]
 use libc::__rlimit_resource_t;
+use libc::id_t;
 use libc::pid_t;
 use libc::CLD_EXITED;
 use libc::EINTR;
 use libc::ESRCH;
+use libc::EXIT_SUCCESS;
 use libc::P_PID;
 use libc::SIGKILL;
 use libc::WEXITED;
@@ -31,9 +34,18 @@ if_memory_limit! {
     use libc::RLIMIT_AS;
 }
 
-#[cfg(any(target_env = "musl", target_os = "android"))]
+macro_rules! static_assert {
+    ( $condition:expr ) => {
+        const _: () = [()][if $condition { 0 } else { 1 }];
+    };
+}
+
+#[cfg(any(
+    all(target_env = "musl", target_os = "linux"),
+    target_os = "android",
+))]
 type LimitResource = c_int;
-#[cfg(target_env = "gnu")]
+#[cfg(all(target_env = "gnu", target_os = "linux"))]
 type LimitResource = __rlimit_resource_t;
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
@@ -45,7 +57,7 @@ pub(super) struct ExitStatus {
 
 impl ExitStatus {
     pub(super) const fn success(self) -> bool {
-        self.exited && self.value == 0
+        self.exited && self.value == EXIT_SUCCESS
     }
 
     fn get_value(self, exit: bool) -> Option<c_int> {
@@ -105,6 +117,12 @@ where
     F: 'static + FnOnce() -> R + Send,
     R: 'static + Send,
 {
+    let time_limit = if let Some(time_limit) = time_limit {
+        time_limit
+    } else {
+        return Ok(Some(run_fn()));
+    };
+
     let (result_sender, result_receiver) = {
         #[cfg(feature = "crossbeam-channel")]
         {
@@ -117,26 +135,28 @@ where
             mpsc::channel()
         }
     };
-    let _ =
-        thread::Builder::new().spawn(move || result_sender.send(run_fn()))?;
 
-    Ok(time_limit
-        .map(|x| result_receiver.recv_timeout(x).ok())
-        .unwrap_or_else(|| {
-            Some(result_receiver.recv().expect("channel was disconnected"))
-        }))
+    thread::Builder::new()
+        .spawn(move || result_sender.send(run_fn()))
+        .map(|_| result_receiver.recv_timeout(time_limit).ok())
 }
 
+const INVALID_PID_ERROR: &str = "process identifier is invalid";
+
 #[derive(Debug)]
-struct RawPid(u32);
+struct RawPid(pid_t);
 
 impl RawPid {
     fn new(process: &Child) -> Self {
-        Self(process.id())
+        let pid: u32 = process.id();
+        Self(pid.try_into().expect(INVALID_PID_ERROR))
     }
 
-    fn as_pid(&self) -> pid_t {
-        self.0.try_into().expect("process identifier is invalid")
+    const fn as_id(&self) -> id_t {
+        static_assert!(pid_t::MAX == i32::MAX);
+        static_assert!(mem::size_of::<pid_t>() <= mem::size_of::<id_t>());
+
+        self.0 as _
     }
 }
 
@@ -144,9 +164,11 @@ impl RawPid {
 pub(super) struct SharedHandle {
     pid: RawPid,
     #[cfg(any(
-        target_env = "gnu",
-        target_env = "musl",
         target_os = "android",
+        all(
+            target_os = "linux",
+            any(target_env = "gnu", target_env = "musl"),
+        ),
     ))]
     pub(super) memory_limit: Option<usize>,
     pub(super) time_limit: Option<Duration>,
@@ -157,9 +179,11 @@ impl SharedHandle {
         Self {
             pid: RawPid::new(process),
             #[cfg(any(
-                target_env = "gnu",
-                target_env = "musl",
                 target_os = "android",
+                all(
+                    target_os = "linux",
+                    any(target_env = "gnu", target_env = "musl"),
+                ),
             ))]
             memory_limit: None,
             time_limit: None,
@@ -191,7 +215,7 @@ impl SharedHandle {
                 .into();
 
             check_syscall(libc::prlimit(
-                self.pid.as_pid(),
+                self.pid.0,
                 resource,
                 &rlimit {
                     rlim_cur: limit,
@@ -206,9 +230,11 @@ impl SharedHandle {
         // https://github.com/rust-lang/rust/blob/49c68bd53f90e375bfb3cbba8c1c67a9e0adb9c0/src/libstd/sys/unix/process/process_unix.rs#L432-L441
 
         #[cfg(any(
-            target_env = "gnu",
-            target_env = "musl",
             target_os = "android",
+            all(
+                target_os = "linux",
+                any(target_env = "gnu", target_env = "musl"),
+            ),
         ))]
         if let Some(memory_limit) = self.memory_limit {
             unsafe {
@@ -216,18 +242,14 @@ impl SharedHandle {
             }
         }
 
-        let pid = self.pid.0;
+        let pid = self.pid.as_id();
         run_with_time_limit(
             move || loop {
                 let mut process_info = MaybeUninit::uninit();
-                #[cfg_attr(
-                    not(target_os = "freebsd"),
-                    allow(clippy::useless_conversion)
-                )]
                 let result = check_syscall(unsafe {
                     libc::waitid(
                         P_PID,
-                        pid.into(),
+                        pid,
                         process_info.as_mut_ptr(),
                         WEXITED | WNOWAIT | WSTOPPED,
                     )
@@ -263,8 +285,9 @@ impl DuplicatedHandle {
         Ok(Self(RawPid::new(process)))
     }
 
+    #[rustfmt::skip]
     pub(super) unsafe fn terminate(&self) -> io::Result<()> {
-        check_syscall(libc::kill(self.0.as_pid(), SIGKILL)).map_err(|error| {
+        check_syscall(libc::kill(self.0.0, SIGKILL)).map_err(|error| {
             // This error is usually decoded to [ErrorKind::Other]:
             // https://github.com/rust-lang/rust/blob/49c68bd53f90e375bfb3cbba8c1c67a9e0adb9c0/src/libstd/sys/unix/mod.rs#L100-L123
             if error.raw_os_error() == Some(ESRCH) {
